/*

   Copyright (C) 2013 Andrej Lojdl

   This file is part of Octave.

   Octave is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Octave is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with Octave; see the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.

 */

#if ! defined(txt_render_h)
#define txt_render_h 1

#include <dMatrix.h>
#include <uint8NDArray.h>
#include <string>
#include "txt-eng.h"
#include "caseless-str.h"


class OCTINTERP_API text_render;

class 
OCTINTERP_API
base_text_render
{
public:
  friend class text_render;

  base_text_render (void) : count(1) { }

  virtual ~base_text_render (void) { }

  virtual void set_font (const std::string& name, const std::string& weight,
                         const std::string& angle, double size) = 0;

  virtual void set_color (Matrix c) = 0;

  virtual void text_to_pixels (const std::string& txt,
                               uint8NDArray& pixels_, Matrix& bbox,
                               int halign, int valign, double rotation) = 0;

  virtual Matrix get_extent (text_element *elt, double rotation = 0.0) = 0;
  virtual Matrix get_extent (const std::string& txt, double rotation = 0.0) = 0;

private: 
  octave_refcount<int> count;        
};

/* if user don't have FREETYPE library we use this dummy class */

class
OCTINTERP_API
dummy_render : public base_text_render
{
public:
  dummy_render (void);

  ~dummy_render (void);

  void set_font (const std::string& name, const std::string& weight,
                 const std::string& angle, double size);

  void set_color (Matrix c);

  void text_to_pixels (const std::string& txt,
                       uint8NDArray& pixels_, Matrix& bbox,
                       int halign, int valign, double rotation);

  Matrix get_extent (text_element *elt, double rotation = 0.0);
  Matrix get_extent (const std::string& txt, double rotation = 0.0);

private:
  Matrix bbox;
  uint8NDArray pixels;
  uint8_t red, green, blue;
};


class
OCTINTERP_API
text_render
{
public:
  text_render (const caseless_str& s = caseless_str ())
    : rep (make_text_render(s)) { }

  text_render (const text_render& e) 
    : rep (e.rep)
    {
      rep->count++;
    } 

  ~text_render ( void )
    {
      if( rep && --rep->count == 0 )
        {
          delete rep;
        }
    }

  text_render& operator = (const text_render& e)
    {
      if (rep != e.rep)
        {
          if (rep && --rep->count == 0)
            delete rep;

          rep = e.rep;
          if (rep)
            rep->count++;
        }
      return *this;
    }

  void set_font (const std::string& name, const std::string& weight,
                 const std::string& angle, double size) 
    { 
      rep->set_font (name, weight, angle, size); 
    }


  void set_color (Matrix c) { rep->set_color (c); }

  void text_to_pixels (const std::string& txt,
                       uint8NDArray& pixels_, Matrix& bbox,
                       int halign, int valign, double rotation) 
    {
      rep->text_to_pixels (txt, pixels_, bbox, halign, valign, rotation);
    }

  Matrix get_extent (text_element *elt, double rotation = 0.0)
    {
      return rep->get_extent (elt, rotation);
    }

  Matrix get_extent (const std::string& txt, double rotation = 0.0)
    {
      return rep->get_extent (txt, rotation);
    }


private:
  base_text_render *rep;
  base_text_render *make_text_render (const caseless_str& s);
};
#endif
